/*****************************************************************************************************************************
--
--    File Name    : Median_Filter.v 

--    Description  : This module implements Median Filter for three input samples.

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module Median_Filter #(parameter g_DATAWIDTH = 8)
                     (input wire SYS_CLK_I,
                      input wire RESET_n_I,
                      input wire[g_DATAWIDTH - 1 : 0] A,
                      input wire[g_DATAWIDTH - 1 : 0] B,
                      input wire[g_DATAWIDTH - 1 : 0] C,
                      output reg[g_DATAWIDTH - 1 : 0] M);
  wire oo0;
  wire io0;
  wire Oi0;
  wire[2 : 0] Ii0;
  assign oo0 = ($signed(A) > $signed(B)) ? 1'b1 : 1'b0;
  assign io0 = ($signed(B) > $signed(C)) ? 1'b1 : 1'b0;
  assign Oi0 = ($signed(C) > $signed(A)) ? 1'b1 : 1'b0;
  assign Ii0 = { oo0, io0, Oi0 };
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          M <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          case (Ii0)
            3'd0:
              M <= A;
            3'd1:
              M <= B;
            3'd2:
              M <= A;
            3'd3:
              M <= C;
            3'd4:
              M <= C;
            3'd5:
              M <= A;
            3'd6:
              M <= B;
            3'd7:
              M <= A;
            default:
              ;
          endcase
        end
    end
endmodule
