/*****************************************************************************************************************************
--
--    File Name    : sobel.v 

--    Description  : This module implements Sobel operator .

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

module sobel #(parameter g_DATA_BITWIDTH = 8)
             (input wire SYS_CLK_I,
              input wire nRESET_I,
              input wire[g_DATA_BITWIDTH - 1 : 0] p0_i,
              input wire[g_DATA_BITWIDTH - 1 : 0] p1_i,
              input wire[g_DATA_BITWIDTH - 1 : 0] p2_i,
              input wire[g_DATA_BITWIDTH - 1 : 0] p3_i,
              input wire[g_DATA_BITWIDTH - 1 : 0] p5_i,
              input wire[g_DATA_BITWIDTH - 1 : 0] p6_i,
              input wire[g_DATA_BITWIDTH - 1 : 0] p7_i,
              input wire[g_DATA_BITWIDTH - 1 : 0] p8_i,
              output wire[g_DATA_BITWIDTH - 1 : 0] DATA_OUT_o);
  reg signed[g_DATA_BITWIDTH + 1 : 0] ill;
  reg signed[g_DATA_BITWIDTH + 1 : 0] O0l;
  reg[g_DATA_BITWIDTH + 3 : 0] I0l;
  reg[g_DATA_BITWIDTH - 1 : 0] l0l;
  reg[g_DATA_BITWIDTH - 1 : 0] o0l;
  reg[g_DATA_BITWIDTH - 1 : 0] i0l;
  reg[g_DATA_BITWIDTH - 1 : 0] O1l;
  reg[g_DATA_BITWIDTH - 1 : 0] I1l;
  reg[g_DATA_BITWIDTH - 1 : 0] l1l;
  reg[g_DATA_BITWIDTH - 1 : 0] o1l;
  reg[g_DATA_BITWIDTH - 1 : 0] i1l;
  reg signed[g_DATA_BITWIDTH + 2 : 0] Ool;
  reg signed[g_DATA_BITWIDTH + 2 : 0] Iol;
  reg signed[g_DATA_BITWIDTH + 1 : 0] lol,
                                      ool,
                                      iol,
                                      Oil,
                                      Iil,
                                      lil;
  always
    @(posedge (SYS_CLK_I),
      negedge (nRESET_I))
    begin
      if (!nRESET_I)
        begin
          l0l <= { g_DATA_BITWIDTH { 1'b0 } };
          o0l <= { g_DATA_BITWIDTH { 1'b0 } };
          i0l <= { g_DATA_BITWIDTH { 1'b0 } };
          O1l <= { g_DATA_BITWIDTH { 1'b0 } };
          I1l <= { g_DATA_BITWIDTH { 1'b0 } };
          l1l <= { g_DATA_BITWIDTH { 1'b0 } };
          o1l <= { g_DATA_BITWIDTH { 1'b0 } };
          i1l <= { g_DATA_BITWIDTH { 1'b0 } };
        end
      else
        begin
          l0l <= p0_i;
          o0l <= p1_i;
          i0l <= p2_i;
          O1l <= p3_i;
          I1l <= p5_i;
          l1l <= p6_i;
          o1l <= p7_i;
          i1l <= p8_i;
        end
    end
  always
    @(posedge (SYS_CLK_I),
      negedge (nRESET_I))
    begin
      if (!nRESET_I)
        begin
          lol <= 0;
          ool <= 0;
          iol <= 0;
          Oil <= 0;
          Iil <= 0;
          lil <= 0;
        end
      else
        begin
          lol <= (i0l - l0l);
          ool <= (I1l - O1l);
          iol <= (i1l - l1l);
          Oil <= (l0l - l1l);
          Iil <= (o0l - o1l);
          lil <= (i0l - i1l);
        end
    end
  always
    @(posedge (SYS_CLK_I),
      negedge (nRESET_I))
    begin
      if (!nRESET_I)
        begin
          ill <= { g_DATA_BITWIDTH + 2 { 1'b0 } };
          O0l <= { g_DATA_BITWIDTH + 2 { 1'b0 } };
        end
      else
        begin
          ill <= (lol + (ool << 1) + iol);
          O0l <= (Oil + (Iil << 1) + lil);
        end
    end
  always
    @(*)
    begin
      if (ill[g_DATA_BITWIDTH + 1])
        begin
          Ool <= ~ill + 1;
        end
      else
        begin
          Ool <= { ill[g_DATA_BITWIDTH + 1], ill };
        end
    end
  always
    @(*)
    begin
      if (O0l[g_DATA_BITWIDTH + 1])
        begin
          Iol <= ~O0l + 1;
        end
      else
        begin
          Iol <= { O0l[g_DATA_BITWIDTH + 1], O0l };
        end
    end
  always
    @(posedge (SYS_CLK_I),
      negedge (nRESET_I))
    begin
      if (!nRESET_I)
        begin
          I0l <= { g_DATA_BITWIDTH + 4 { 1'b0 } };
        end
      else
        begin
          I0l <= (Ool + Iol);
        end
    end
  assign DATA_OUT_o = (|I0l[11 : 8]) ? 8'hff : I0l[7 : 0];
endmodule
