/*****************************************************************************************************************************
--
--    File Name    : YCbCr2RGB_tb.v 

--    Description  : This module provides the test environment for YCbCr to RGB IP.

					 The following file is required for the simulation :
					 YCbCr_in.txt -->  It contains hex values which are in the following format: 
									 Cr,Cb and Y values
									 For more details visit Microsemi Video Web Page.

--                   

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module YCbCr2RGB_test;
  /************************************************************************
          Parameters 
  *************************************************************************/
  parameter HEIGHT = 200;
  parameter WIDTH = 320;
  parameter CLKPERIOD = 10;
  parameter DATA_BIT_WIDTH = 24;
  parameter IMAGE_FILE_NAME = "YCbCr_in.txt";  
  //Parameters from the DUT
  parameter g_RGB_DATA_BIT_WIDTH = 8;
  parameter g_YCbCr_DATA_BIT_WIDTH = 8;
  localparam MAX_SIZE = (HEIGHT * WIDTH * (DATA_BIT_WIDTH / g_RGB_DATA_BIT_WIDTH)) + 4;
  localparam MAX_SIZE_24 = (HEIGHT * WIDTH) + 1;
  /************************************************************************
          Inputs and Outputs to the DUT 
  *************************************************************************/
  reg Clock;
  reg Reset;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Y;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cb;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cr;
  reg dataValidIn_y2r;
  reg H_sync_in;
  reg V_sync_in;
  reg H_active_in;
  reg V_active_in;
  wire[g_RGB_DATA_BIT_WIDTH - 1 : 0] Red_out;
  wire[g_RGB_DATA_BIT_WIDTH - 1 : 0] Green_out;
  wire[g_RGB_DATA_BIT_WIDTH - 1 : 0] Blue_out;
  wire dataValidOut_y2r;
  wire H_sync_out;
  wire V_sync_out;
  wire H_active_out;
  wire V_active_out;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] image_8[MAX_SIZE - 5 : 0];
  reg[DATA_BIT_WIDTH - 1 : 0] image_24[MAX_SIZE_24 - 2 : 0];
  integer i,
          j;
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] Mem_B[MAX_SIZE_24 - 2 : 0];
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] Mem_G[MAX_SIZE_24 - 2 : 0];
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] Mem_R[MAX_SIZE_24 - 2 : 0];
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] Mem_final[MAX_SIZE - 2 : 0];
  integer k_y2r,
          l,
          m,
	  File_1;
  reg[31 : 0] counter_h_y2r,
              counter_v_y2r,
              counter_h_in_y2r,
              counter_v_in_y2r;
  reg[2 : 0] counter_line_wait;
  reg[3 : 0] count_done_y2r;
  reg[1 : 0] state_y2r;
  localparam[1 : 0] IDLE_y2r = 0,
                    READ_y2r = 1,
                    WAIT_y2r = 2,
                    DONE_y2r = 3;
  initial
    begin
      Reset = 0;
      Clock = 0;
      H_sync_in = 0;
      V_sync_in = 0;
      H_active_in = 0;
      V_active_in = 0;
      Y = 0;
      Cb = 0;
      Cr = 0;
      k_y2r = 0;
      l = 0;
      m = 0;
      j = 0;
      counter_v_y2r = 0;
      counter_h_y2r = 0;
      counter_v_in_y2r = 0;
      counter_h_in_y2r = 0;
      count_done_y2r = 0;
      counter_line_wait = 0;
      state_y2r = 0;
    end
  /************************************************************************
          Generates Clock signal
  *************************************************************************/
  initial
    begin
      forever
        #(CLKPERIOD / 2)
        Clock = ~Clock;
    end
  /************************************************************************
          RESET : Generates a Reset pulse 
  *************************************************************************/
  initial
    begin
     File_1 = $fopen("RGB_out.txt","wb");
      Reset = 1;
      #4
      ;
      Reset = 0;
      #2
      ;
      Reset = 1;
    end
  /************************************************************************
          Read the input image 
  *************************************************************************/
  initial
    begin
      $readmemh(IMAGE_FILE_NAME, image_8);
      for (i = 0; i < MAX_SIZE_24; i = i + 1)
        begin
          image_24[i] = (({ 16'b0, image_8[i * 3] } << 16) | ({ 16'b0, image_8[i * 3 + 1] } << 8) | { image_8[i * 3 + 2] });
        end
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>                     IMAGE LOADED                                           <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
    end
  /************************************************************************
          Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  always
    @(posedge Clock)
    begin
      if (counter_h_y2r == (WIDTH))
        begin
          counter_h_y2r <= 0;
          counter_v_y2r <= counter_v_y2r + 1;
        end
      else
        if (dataValidOut_y2r)
          counter_h_y2r <= counter_h_y2r + 1;
    end
  /************************************************************************
          Debug Signals: Input vertical counter
  *************************************************************************/
  always
    @(posedge Clock)
    begin
      if ((Reset == 1) && (counter_v_in_y2r < HEIGHT) && (counter_line_wait == 2'd1))
        counter_v_in_y2r <= counter_v_in_y2r + 1;
      else
        begin
          counter_v_in_y2r <= counter_v_in_y2r;
        end
    end
  /******************************************************************************
          FSM to process the input image 
  *******************************************************************************/
  always
    @(posedge Clock)
    begin
      if (!Reset)
        begin
          state_y2r = 0;
          count_done_y2r = 0;
        end
      else
        begin
          case (state_y2r)
            IDLE_y2r:
              begin
                $display("");
                $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                $display(">>>>>>>>                    PROCESSING OF IMAGE STARTED                             <<<<<<<");
                $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                state_y2r <= READ_y2r;
              end
            READ_y2r:
              begin
                if ((counter_h_in_y2r == (WIDTH)) && (Reset == 1))
                  begin
                    dataValidIn_y2r <= 0;
                    state_y2r <= WAIT_y2r;
                    H_sync_in <= 1;
                    V_sync_in <= 1;
                    H_active_in <= 1;
                    V_active_in <= 1;
                  end
                else
                  begin
                    if ((counter_h_in_y2r < (WIDTH)) && (Reset == 1))
                      begin
                        H_sync_in <= 0;
                        V_sync_in <= 0;
                        H_active_in <= 0;
                        V_active_in <= 0;
                        dataValidIn_y2r <= 1;
                        Y <= image_24[j][7 : 0];
                        Cb <= image_24[j][15 : 8];
                        Cr <= image_24[j][23 : 16];
                        j <= j + 1;
                        counter_h_in_y2r <= counter_h_in_y2r + 1;
                        state_y2r <= READ_y2r;
                      end
                    else
                      state_y2r <= READ_y2r;
                  end
              end
            WAIT_y2r:
              begin
                if (counter_v_in_y2r == HEIGHT)
                  begin
                    dataValidIn_y2r <= 0;
                    state_y2r <= DONE_y2r;
                    counter_v_y2r <= counter_v_y2r;
                    H_sync_in <= 0;
                    V_sync_in <= 0;
                    H_active_in <= 0;
                    V_active_in <= 0;
                  end
                else
                  begin
                    if (counter_line_wait == 2'd3)
                      begin
                        counter_line_wait <= 0;
                        counter_h_in_y2r <= 0;
                        state_y2r <= READ_y2r;
                        H_sync_in <= 0;
                        V_sync_in <= 0;
                        H_active_in <= 0;
                        V_active_in <= 0;
                      end
                    else
                      begin
                        counter_line_wait <= counter_line_wait + 1;
                        state_y2r <= WAIT_y2r;
                        H_sync_in <= 0;
                        V_sync_in <= 0;
                        H_active_in <= 0;
                        V_active_in <= 0;
                      end
                  end
              end
            DONE_y2r:
              begin
                if (count_done_y2r == 4'd15)
                  begin
                    $display("");
                    $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                    $display(">>>>>>>>        \t\t\tSIMULATION COMPLETED\t    \t\t\t\t\t\t      <<<<<<<");
                    $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                    $finish;
                  end
                else
                  begin
                    if (counter_v_y2r == HEIGHT)
                      begin
                        count_done_y2r <= count_done_y2r + 1;
                        state_y2r <= DONE_y2r;
                      end
                  end
              end
          endcase
        end
    end
  /************************************************************************
          Process to write the output RGB into a memory
  *************************************************************************/
  always
    @(posedge Clock)
    begin
      if (dataValidOut_y2r)
        begin
	  $fwrite(File_1,"%x\n",Blue_out);
	  $fwrite(File_1,"%x\n",Green_out);
	  $fwrite(File_1,"%x\n",Red_out);
          k_y2r <= k_y2r + 1;
        end
    end

  /************************************************************************
  Component instantiation
  *************************************************************************/
  YCbCr2RGB #(.g_RGB_DATA_BIT_WIDTH(g_RGB_DATA_BIT_WIDTH), .g_YCbCr_DATA_BIT_WIDTH(g_YCbCr_DATA_BIT_WIDTH)) Y2R_Converter_0(.CLOCK_I(Clock),
                                                                                                                            .RESET_N_I(Reset),
                                                                                                                            .Y_i(Y),
                                                                                                                            .Cb_i(Cb),
                                                                                                                            .Cr_i(Cr),
                                                                                                                            .data_enable_i(dataValidIn_y2r),
                                                                                                                            .horz_sync_i(H_sync_in),
                                                                                                                            .vert_sync_i(V_sync_in),
                                                                                                                            .hactive_i(H_active_in),
                                                                                                                            .vactive_i(V_active_in),
                                                                                                                            .Red_out_o(Red_out),
                                                                                                                            .Green_out_o(Green_out),
                                                                                                                            .Blue_out_o(Blue_out),
                                                                                                                            .data_enable_o(dataValidOut_y2r),
                                                                                                                            .horz_sync_o(H_sync_out),
                                                                                                                            .vert_sync_o(V_sync_out),
                                                                                                                            .hactive_o(H_active_out),
                                                                                                                            .vactive_o(V_active_out));
endmodule
