/*****************************************************************************************************************************
--
--    File Name    : AXI_M.v

--    Description  : This module implements AXI wrapper
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

********************************************************************************************************************************/
`timescale 1ns/1ps
module AXI_M #(parameter g_AXI_AWIDTH = 32,
               parameter g_AXI_DWIDTH = 64)
             (input wire clock,
              input wire nreset,
              output reg[3 : 0] m_awid,
              output reg[(g_AXI_AWIDTH - 1) : 0] m_awaddr,
              output reg[3 : 0] m_awlen,
              output reg[2 : 0] m_awsize,
              output reg[1 : 0] m_awburst,
              output wire[1 : 0] m_awlock,
              output wire[3 : 0] m_awcache,
              output wire[2 : 0] m_awprot,
              output reg m_awvalid,
              input wire m_awready,
              output reg[3 : 0] m_wid,
              output reg[(g_AXI_DWIDTH - 1) : 0] m_wdata,
              output wire[7 : 0] m_wstrb,
              output reg m_wlast,
              output reg m_wvalid,
              input wire m_wready,
              input wire[3 : 0] m_bid,
              input wire[1 : 0] m_bresp,
              input wire m_bvalid,
              output reg m_bready,
              output reg[3 : 0] m_arid,
              output reg[(g_AXI_AWIDTH - 1) : 0] m_araddr,
              output reg[3 : 0] m_arlen,
              output reg[2 : 0] m_arsize,
              output reg[1 : 0] m_arburst,
              output wire[1 : 0] m_arlock,
              output wire[3 : 0] m_arcache,
              output wire[2 : 0] m_arprot,
              output reg m_arvalid,
              input wire m_arready,
              input wire[3 : 0] m_rid,
              input wire[(g_AXI_DWIDTH - 1) : 0] m_rdata,
              input wire[1 : 0] m_rresp,
              input wire m_rlast,
              input wire m_rvalid,
              output reg m_rready,
              input wire[3 : 0] tr_length_w,
              input wire[2 : 0] tr_size_w,
              input wire[1 : 0] tr_burst_w,
              input wire[(g_AXI_AWIDTH - 1) : 0] tr_daddr_w,
              input wire[1 : 0] tr_type_w,
              input wire tr_trg_w,
              output reg tr_done_w,
              input wire[3 : 0] tr_length_r,
              input wire[2 : 0] tr_size_r,
              input wire[1 : 0] tr_burst_r,
              input wire[(g_AXI_AWIDTH - 1) : 0] tr_saddr_r,
              input wire[1 : 0] tr_type_r,
              input wire tr_trg_r,
              output reg tr_done_r,
              output wire[(g_AXI_DWIDTH - 1) : 0] rdata_dout,
              output reg buff_we,
              input wire[(g_AXI_DWIDTH - 1) : 0] wdata_din,
              output wire buff_re);
  parameter[7 : 0] O = 8'hff;
  parameter[1 : 0] I = 2'b00;
  parameter[3 : 0] l = 4'b0011;
  parameter[3 : 0] o = 4'b0000;
  parameter[2 : 0] i = 3'b000;
  parameter[2 : 0] OI = 3'b000;
  parameter[1 : 0] II = 2'd0,
                   lI = 2'd1,
                   oI = 2'd2,
                   iI = 2'd3;
  parameter[2 : 0] Ol = 3'd0,
                   Il = 3'd1,
                   ll = 3'd2,
                   ol = 3'd3,
                   il = 3'd4;
  reg[3 : 0] O0;
  reg[2 : 0] I0;
  reg[1 : 0] l0;
  reg[(g_AXI_AWIDTH - 1) : 0] o0;
  reg[1 : 0] i0;
  reg[3 : 0] O1;
  reg[2 : 0] I1;
  reg[1 : 0] l1;
  reg[(g_AXI_AWIDTH - 1) : 0] o1;
  reg[1 : 0] i1;
  reg[2 : 0] Oo;
  reg[1 : 0] Io;
  reg[3 : 0] lo;
  reg[(g_AXI_DWIDTH - 1) : 0] oo;
  reg io;
  reg Oi;
  wire Ii;
  assign m_wstrb = O;
  assign m_awlock = I;
  assign m_arlock = I;
  assign rdata_dout = oo;
  assign m_awcache = l;
  assign m_arcache = o;
  assign m_arprot = i;
  assign m_awprot = OI;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          Oo <= Ol;
        end
      else
        begin
          case (Oo)
            Ol:
              begin
                if ((tr_trg_w == 1'b1) && (tr_type_w == 2'b01))
                  Oo <= Il;
                else
                  Oo <= Ol;
              end
            Il:
              begin
                if ((m_awready == 1'b1) && (m_awvalid == 1'b1))
                  begin
                    Oo <= il;
                  end
                else
                  begin
                    Oo <= Il;
                  end
              end
            il:
              begin
                Oo <= ll;
              end
            ll:
              begin
                if (m_wlast)
                  begin
                    Oo <= ol;
                  end
                else
                  if ((m_wready == 1'b1) && (m_wvalid == 1'b1))
                    begin
                      Oo <= ll;
                    end
                  else
                    begin
                      Oo <= ll;
                    end
              end
            ol:
              begin
                if ((m_bid == 4'b0101) && (m_bvalid == 1'b1) && (m_bresp == 2'b00))
                  begin
                    Oo <= Ol;
                  end
                else
                  if ((m_bid == 4'b0101) && (m_bvalid == 1'b1) && (m_bresp == 2'b10))
                    begin
                      Oo <= Ol;
                    end
                  else
                    begin
                      Oo <= ol;
                    end
              end
            default:
              Oo <= Ol;
          endcase
        end
    end
  assign Ii = m_wvalid;
  assign buff_re = (io) ? 1'b1 : ((m_wlast) ? 1'b0 : ((Oi == m_wvalid) ? 1'b0 : m_wvalid));
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        Oi <= 1'b0;
      else
        Oi <= m_wvalid;
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          O0 <= 4'b0000;
          I0 <= 3'b0;
          l0 <= 2'b0;
          o0 <= { g_AXI_AWIDTH { 1'b0 } };
          i0 <= 2'b00;
          tr_done_w <= 1'b0;
          lo <= 4'b0;
          m_bready <= 1'b0;
          m_wvalid <= 1'b0;
          m_wlast <= 1'b0;
          m_wid <= 4'b0;
          m_wdata <= { g_AXI_DWIDTH { 1'b0 } };
          m_awlen <= 4'b0;
          m_awvalid <= 1'b0;
          m_awsize <= 3'b0;
          m_awid <= 4'b0;
          m_awburst <= 2'b0;
          m_awaddr <= { g_AXI_AWIDTH { 1'b0 } };
          io <= 1'b0;
        end
      else
        begin
          case (Oo)
            Ol:
              begin
                O0 <= tr_length_w;
                I0 <= tr_size_w;
                l0 <= tr_burst_w;
                o0 <= tr_daddr_w;
                i0 <= tr_type_w;
                tr_done_w <= 1'b0;
                m_bready <= 1'b0;
                io <= 1'b0;
              end
            Il:
              begin
                m_awid <= { 2'b01, i0 };
                m_awaddr <= o0;
                m_awlen <= O0;
                m_awsize <= I0;
                m_awburst <= l0;
                tr_done_w <= 1'b0;
                lo <= 4'b0;
                m_wvalid <= 1'b0;
                m_wlast <= 1'b0;
                if ((m_awready == 1'b1) && (m_awvalid == 1'b1))
                  begin
                    m_awvalid <= 1'b0;
                    io <= 1'b1;
                  end
                else
                  begin
                    m_awvalid <= 1'b1;
                  end
              end
            il:
              begin
                io <= 1'b0;
              end
            ll:
              begin
                io <= 1'b0;
                if ((m_wvalid == 1'b1) && (m_wready == 1'b0))
                  m_wdata <= m_wdata;
                else
                  m_wdata <= wdata_din;
                m_wid <= { 2'b01, i0 };
                if ((lo == O0) && (m_wready == 1'b1))
                  begin
                    lo <= 4'b0;
                  end
                else
                  if ((m_wready == 1'b1) && (m_wvalid == 1'b1))
                    begin
                      lo <= lo + 4'b1;
                    end
                  else
                    begin
                      lo <= lo;
                    end
                if ((lo == O0))
                  m_wlast <= 1'b1;
                else
                  m_wlast <= m_wlast;
                if ((m_wready == 1'b1) && (m_wvalid == 1'b1) && (lo == O0))
                  begin
                    m_wvalid <= 1'b0;
                  end
                else
                  if (m_wready && m_wvalid)
                    m_wvalid <= 1'b0;
                  else
                    begin
                      m_wvalid <= 1'b1;
                    end
              end
            ol:
              begin
                m_wlast <= 1'b0;
                m_wvalid <= 1'b0;
                if ((m_bid == 4'b0101) && (m_bvalid == 1'b1) && (m_bresp == 2'b00))
                  begin
                    m_bready <= 1'b1;
                    tr_done_w <= 1'b1;
                  end
                else
                  if ((m_bid == 4'b0101) && (m_bvalid == 1'b1) && (m_bresp == 2'b10))
                    begin
                      m_bready <= 1'b1;
                      tr_done_w <= 1'b1;
                    end
                  else
                    begin
                      m_bready <= 1'b0;
                      tr_done_w <= 1'b0;
                    end
              end
            default:
              begin
              end
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          Io <= II;
        end
      else
        begin
          case (Io)
            II:
              begin
                if ((tr_trg_r == 1'b1) && (tr_type_r == 2'b00))
                  Io <= lI;
                else
                  Io <= II;
              end
            lI:
              begin
                if ((m_arready == 1'b1) && (m_arvalid == 1'b1))
                  begin
                    Io <= oI;
                  end
                else
                  begin
                    Io <= lI;
                  end
              end
            oI:
              begin
                if ((m_rvalid == 1'b1) && (m_rlast == 1'b1) && (m_rid == m_arid))
                  begin
                    Io <= II;
                  end
                else
                  begin
                    Io <= oI;
                  end
              end
            iI:
              begin
                Io <= oI;
              end
            default:
              Io <= II;
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          O1 <= 4'b0000;
          I1 <= 3'b0;
          l1 <= 2'b0;
          o1 <= { g_AXI_AWIDTH { 1'b0 } };
          i1 <= 2'b00;
          tr_done_r <= 1'b0;
          oo <= { g_AXI_DWIDTH { 1'b0 } };
          m_rready <= 1'b0;
          m_arvalid <= 1'b0;
          m_arsize <= 3'b0;
          m_arlen <= 4'b0;
          m_arid <= 4'b0;
          m_arburst <= 2'b0;
          m_araddr <= { g_AXI_AWIDTH { 1'b0 } };
          buff_we <= 1'b0;
        end
      else
        begin
          case (Io)
            II:
              begin
                O1 <= tr_length_r;
                I1 <= tr_size_r;
                l1 <= tr_burst_r;
                o1 <= tr_saddr_r;
                i1 <= tr_type_r;
                tr_done_r <= 1'b0;
                m_rready <= 1'b0;
                buff_we <= 1'b0;
                m_arvalid <= 1'b0;
              end
            lI:
              begin
                m_arid <= { 2'b01, i1 };
                m_araddr <= o1;
                m_arlen <= O1;
                m_arsize <= I1;
                m_arburst <= l1;
                tr_done_r <= 1'b0;
                buff_we <= 1'b0;
                if ((m_arready == 1'b1) && (m_arvalid == 1'b1))
                  begin
                    m_arvalid <= 1'b0;
                  end
                else
                  begin
                    m_arvalid <= 1'b1;
                  end
              end
            oI:
              begin
                if ((m_rvalid == 1'b1) && (m_rlast == 1'b1) && (m_rid == m_arid))
                  begin
                    m_rready <= 1'b1;
                    oo <= m_rdata;
                    buff_we <= 1'b1;
                    tr_done_r <= 1'b1;
                  end
                else
                  if (m_rvalid == 1'b1)
                    begin
                      m_rready <= 1'b1;
                      oo <= m_rdata;
                      buff_we <= 1'b1;
                      tr_done_r <= 1'b0;
                    end
                  else
                    begin
                      m_rready <= 1'b1;
                      oo <= m_rdata;
                      buff_we <= 1'b0;
                      tr_done_r <= 1'b0;
                    end
              end
            iI:
              begin
                m_rready <= 1'b1;
                buff_we <= 1'b0;
                tr_done_r <= 1'b0;
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
