/*****************************************************************************************************************************
--
--    File Name    : data_packer_24_64.v

--    Description  : This module implements 24-bit to 64-bit packer for write channels 1 and 2
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module pack_24_64 #(parameter g_AXI_DWIDTH = 64,
                    parameter g_AXI_BUFF_AWIDTH = 10,
                    parameter g_WR_CHANNEL_VIDEO_DATA_WIDTH = 24)
                  (input wire clock,
                   input wire nreset,
                   input wire start_pack,
                   input wire[(g_WR_CHANNEL_VIDEO_DATA_WIDTH - 1) : 0] video_data_in,
                   input wire data_valid,
                   output wire buff_we,
                   output reg[(g_AXI_BUFF_AWIDTH - 1) : 0] buff_waddr,
                   output wire[(g_AXI_DWIDTH - 1) : 0] buff_data_o);
  reg[(g_AXI_DWIDTH - 1) : 0] oll;
  reg[15 : 0] ill;
  reg O0l;
  reg I0l;
  parameter[2 : 0] l0l = 4'd0,
                   o0l = 4'd1,
                   i0l = 4'd2,
                   O1l = 4'd3,
                   I1l = 4'd4,
                   l1l = 4'd5,
                   o1l = 4'd6,
                   i1l = 4'd7;
  reg[2 : 0] Ool;
  assign buff_data_o = oll;
  assign buff_we = O0l;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        Ool <= l0l;
      else
        if (start_pack)
          begin
            if (data_valid)
              begin
                case (Ool)
                  l0l:
                    begin
                      Ool <= o0l;
                    end
                  o0l:
                    begin
                      Ool <= i0l;
                    end
                  i0l:
                    begin
                      Ool <= O1l;
                    end
                  O1l:
                    begin
                      Ool <= I1l;
                    end
                  I1l:
                    begin
                      Ool <= l1l;
                    end
                  l1l:
                    begin
                      Ool <= o1l;
                    end
                  o1l:
                    begin
                      Ool <= i1l;
                    end
                  i1l:
                    begin
                      Ool <= l0l;
                    end
                  default:
                    Ool <= l0l;
                endcase
              end
            else
              begin
                Ool <= Ool;
              end
          end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          oll <= { g_AXI_DWIDTH { 1'b0 } };
          ill <= 16'd0;
          buff_waddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          O0l <= 1'b0;
          I0l <= 1'b1;
        end
      else
        if (start_pack)
          begin
            if (data_valid)
              begin
                case (Ool)
                  l0l:
                    begin
                      oll <= { video_data_in, oll[63 : 24] };
                      O0l <= 1'b0;
                    end
                  o0l:
                    begin
                      oll <= { video_data_in, oll[63 : 24] };
                      O0l <= 1'b0;
                    end
                  i0l:
                    begin
                      O0l <= 1'b1;
                      oll <= { video_data_in[15 : 0], oll[63 : 16] };
                      ill <= { 8'b0, video_data_in[23 : 16] };
                      if (I0l)
                        begin
                          I0l <= 1'b0;
                          buff_waddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                        end
                      else
                        begin
                          I0l <= 1'b0;
                          buff_waddr <= buff_waddr + 1'b1;
                        end
                    end
                  O1l:
                    begin
                      oll <= { video_data_in, ill[7 : 0], oll[63 : 32] };
                      O0l <= 1'b0;
                    end
                  I1l:
                    begin
                      oll <= { video_data_in, oll[63 : 24] };
                      O0l <= 1'b0;
                    end
                  l1l:
                    begin
                      oll <= { video_data_in[7 : 0], oll[63 : 8] };
                      ill <= { video_data_in[23 : 8] };
                      buff_waddr <= buff_waddr + 1'b1;
                      O0l <= 1'b1;
                    end
                  o1l:
                    begin
                      oll <= { video_data_in, ill, oll[63 : 40] };
                      O0l <= 1'b0;
                    end
                  i1l:
                    begin
                      oll <= { video_data_in, oll[63 : 24] };
                      buff_waddr <= buff_waddr + 1'b1;
                      O0l <= 1'b1;
                    end
                  default:
                    begin
                    end
                endcase
              end
            else
              begin
                oll <= oll;
                O0l <= 1'b0;
                I0l <= I0l;
              end
          end
        else
          begin
            oll <= { g_AXI_DWIDTH { 1'b0 } };
            O0l <= 1'b0;
            ill <= 16'd0;
            buff_waddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
            I0l <= 1'b1;
          end
    end
endmodule
