/*****************************************************************************************************************************
--
--    File Name    : async_fifo_display.v

--    Description  : This module implements the asynchronous video fifo used to store display video pixels
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

`timescale 1ps/1ps
module video_fifo #(parameter g_VIDEO_FIFO_AWIDTH = 12,
                    parameter g_INPUT_VIDEO_DATA_BIT_WIDTH = 24,
                    parameter g_HALF_EMPTY_THRESHOLD = 1280)
                  (input wire wclock_i,
                   input wire wresetn_i,
                   input wire wen_i,
                   input wire[(g_INPUT_VIDEO_DATA_BIT_WIDTH - 1) : 0] wdata_i,
                   output wire wfull_o,
                   output wire wafull_o,
                   output wire[(g_VIDEO_FIFO_AWIDTH - 1) : 0] wdata_count_o,
                   input wire rclock_i,
                   input wire rresetn_i,
                   input wire ren_i,
                   output wire[(g_INPUT_VIDEO_DATA_BIT_WIDTH - 1) : 0] rdata_o,
                   output wire rempty_o,
                   output wire raempty_o,
                   output wire rhempty_o,
                   output wire[(g_VIDEO_FIFO_AWIDTH - 1) : 0] rdata_count_o);
  parameter O = 5900;
  parameter I = 10;
  reg[g_VIDEO_FIFO_AWIDTH : 0] l;
  reg[g_VIDEO_FIFO_AWIDTH : 0] o;
  reg[g_VIDEO_FIFO_AWIDTH : 0] i;
  reg[g_VIDEO_FIFO_AWIDTH : 0] OI;
  reg II;
  reg lI;
  reg[g_VIDEO_FIFO_AWIDTH : 0] oI;
  reg[g_VIDEO_FIFO_AWIDTH : 0] iI;
  reg[g_VIDEO_FIFO_AWIDTH : 0] Ol;
  reg[g_VIDEO_FIFO_AWIDTH : 0] Il;
  reg[g_VIDEO_FIFO_AWIDTH : 0] ll;
  reg[g_VIDEO_FIFO_AWIDTH : 0] ol;
  reg il;
  reg O0;
  reg I0;
  reg[g_VIDEO_FIFO_AWIDTH : 0] l0;
  reg[g_VIDEO_FIFO_AWIDTH : 0] o0;
  wire[g_VIDEO_FIFO_AWIDTH - 1 : 0] i0;
  wire[g_VIDEO_FIFO_AWIDTH : 0] O1;
  wire[g_VIDEO_FIFO_AWIDTH : 0] I1;
  wire[g_VIDEO_FIFO_AWIDTH - 1 : 0] l1;
  wire[g_VIDEO_FIFO_AWIDTH : 0] o1;
  wire[g_VIDEO_FIFO_AWIDTH : 0] i1;
  wire Oo;
  wire[g_VIDEO_FIFO_AWIDTH : 0] Io;
  wire[g_VIDEO_FIFO_AWIDTH : 0] lo;
  integer oo;
  integer io;
  assign rdata_count_o = iI[(g_VIDEO_FIFO_AWIDTH - 1) : 0];
  assign wfull_o = il;
  assign wafull_o = O0;
  assign rhempty_o = I0;
  assign wdata_count_o = o0[(g_VIDEO_FIFO_AWIDTH - 1) : 0];
  assign i0 = l[g_VIDEO_FIFO_AWIDTH - 1 : 0];
  assign O1 = l + (ren_i & (~II));
  assign I1 = (O1 >> 1) ^ O1;
  assign Oo = ((wen_i == 1'b1) && (il == 1'b0)) ? 1'b1 : 1'b0;
  assign l1 = Ol[g_VIDEO_FIFO_AWIDTH - 1 : 0];
  assign o1 = Ol + (wen_i & (~il));
  assign i1 = (o1 >> 1) ^ o1;
  ram2port #(.g_BUFF_AWIDTH(g_VIDEO_FIFO_AWIDTH), .g_DWIDTH(g_INPUT_VIDEO_DATA_BIT_WIDTH)) Oi(.wr_data_i(wdata_i),
                                                                                              .rd_addr(i0),
                                                                                              .wr_addr(l1),
                                                                                              .we(Oo),
                                                                                              .wclock(wclock_i),
                                                                                              .rclock(rclock_i),
                                                                                              .rd_data_o(rdata_o));
  always
    @(posedge rclock_i or
      negedge rresetn_i)
    begin
      if (!rresetn_i)
        begin
          l <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
          o <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
        end
      else
        begin
          l <= O1;
          o <= I1;
        end
    end
  always
    @(posedge rclock_i or
      negedge rresetn_i)
    begin
      if (!rresetn_i)
        begin
          i <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
          OI <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
        end
      else
        begin
          i <= Il;
          OI <= i;
        end
    end
  always
    @(posedge rclock_i or
      negedge rresetn_i)
    begin
      if (!rresetn_i)
        begin
          II <= 1'b1;
        end
      else
        if (I1 == OI)
          begin
            II <= 1'b1;
          end
        else
          begin
            II <= 1'b0;
          end
    end
  assign rempty_o = II;
  always
    @(OI)
    begin
      for (oo = 0; oo <= g_VIDEO_FIFO_AWIDTH; oo = oo + 1)
        begin
          oI[oo] = ^(OI >> oo);
        end
    end
  always
    @(posedge rclock_i or
      negedge rresetn_i)
    begin
      if (!rresetn_i)
        begin
          iI <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
        end
      else
        begin
          iI <= (oI - l);
        end
    end
  assign lo = (oI - l);
  assign raempty_o = lI;
  always
    @(posedge rclock_i or
      negedge rresetn_i)
    begin
      if (!rresetn_i)
        begin
          lI <= 1'b1;
        end
      else
        if (I >= lo)
          begin
            lI <= 1'b1;
          end
        else
          begin
            lI <= 1'b0;
          end
    end
  always
    @(posedge rclock_i or
      negedge rresetn_i)
    begin
      if (!rresetn_i)
        begin
          I0 <= 1'b1;
        end
      else
        if (g_HALF_EMPTY_THRESHOLD >= lo)
          begin
            I0 <= 1'b1;
          end
        else
          begin
            I0 <= 1'b0;
          end
    end
  always
    @(posedge wclock_i or
      negedge wresetn_i)
    begin
      if (!wresetn_i)
        begin
          Ol <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
          Il <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
        end
      else
        begin
          Ol <= o1;
          Il <= i1;
        end
    end
  always
    @(posedge wclock_i or
      negedge wresetn_i)
    begin
      if (!wresetn_i)
        begin
          ll <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
          ol <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
        end
      else
        begin
          ll <= o;
          ol <= ll;
        end
    end
  always
    @(posedge wclock_i or
      negedge wresetn_i)
    begin
      if (!wresetn_i)
        begin
          il <= 1'b0;
        end
      else
        if (i1 == { (~ol[g_VIDEO_FIFO_AWIDTH : g_VIDEO_FIFO_AWIDTH - 1]), ol[g_VIDEO_FIFO_AWIDTH - 2 : 0] })
          begin
            il <= 1'b1;
          end
        else
          begin
            il <= 1'b0;
          end
    end
  always
    @(ol)
    begin
      for (io = 0; io <= g_VIDEO_FIFO_AWIDTH; io = io + 1)
        begin
          l0[io] = ^(ol >> io);
        end
    end
  always
    @(posedge wclock_i or
      negedge wresetn_i)
    begin
      if (!wresetn_i)
        begin
          o0 <= { (g_VIDEO_FIFO_AWIDTH + 1) { 1'b0 } };
        end
      else
        begin
          o0 <= (Ol - l0);
        end
    end
  assign Io = (Ol - l0);
  always
    @(posedge wclock_i or
      negedge wresetn_i)
    begin
      if (!wresetn_i)
        begin
          O0 <= 1'b0;
        end
      else
        if (Io >= O)
          begin
            O0 <= 1'b1;
          end
        else
          begin
            O0 <= 1'b0;
          end
    end
endmodule
