/*****************************************************************************************************************************
--
--    File Name    : cdcfiforam.v

--    Description  : This module implements CDC FIFO RAM                                  

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module cdcfiforam(data,
                  wren,
                  wraddress,
                  rdaddress,
                  wrclock,
                  rdclock,
                  q);
  parameter ADDR_WIDTH = 4;
  parameter DATA_WIDTH = 32;
  input[(DATA_WIDTH - 1) : 0] data;
  input wren;
  input[(ADDR_WIDTH - 1) : 0] wraddress;
  input[(ADDR_WIDTH - 1) : 0] rdaddress;
  input wrclock;
  input rdclock;
  output[(DATA_WIDTH - 1) : 0] q;
  wire[(DATA_WIDTH - 1) : 0] q;
  reg[DATA_WIDTH - 1 : 0] i0l[0 : 2 ** ADDR_WIDTH - 1];
  reg[ADDR_WIDTH - 1 : 0] loI;
  reg[ADDR_WIDTH - 1 : 0] IiI;
  reg O1l;
  reg[DATA_WIDTH - 1 : 0] I1l;
  always
    @(posedge wrclock)
    begin
      O1l <= wren;
      IiI <= wraddress;
      I1l <= data;
      if (O1l == 1'b1)
        begin
          i0l[IiI] <= I1l;
        end
    end
  always
    @(posedge rdclock)
    begin
      loI <= rdaddress;
    end
  assign q = i0l[loI];
endmodule
